@@ -1,9 +1,11 @@
 from __future__ import absolute_import
+from __future__ import with_statement
 
 import pickle
 import sys
 
 from functools import wraps
+from mock import Mock, patch
 
 if sys.version_info >= (3, 0):
     from io import StringIO, BytesIO
@@ -11,9 +13,12 @@ else:
     from StringIO import StringIO, StringIO as BytesIO  # noqa
 
 from kombu import utils
+from kombu.utils.compat import next
 
-from .utils import redirect_stdouts, mask_modules, skip_if_module
-from .utils import TestCase
+from .utils import (
+    TestCase,
+    redirect_stdouts, mask_modules, module_exists, skip_if_module,
+)
 
 
 class OldString(object):
@@ -31,6 +36,13 @@ class OldString(object):
         return self.value.rsplit(*args, **kwargs)
 
 
+class test_kombu_module(TestCase):
+
+    def test_dir(self):
+        import kombu
+        self.assertTrue(dir(kombu))
+
+
 class test_utils(TestCase):
 
     def test_maybe_list(self):
@@ -50,11 +62,11 @@ class test_utils(TestCase):
                          [1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0])
 
     def test_reprkwargs(self):
-        self.assertTrue(utils.reprkwargs({"foo": "bar", 1: 2, u"k": "v"}))
+        self.assertTrue(utils.reprkwargs({'foo': 'bar', 1: 2, u'k': 'v'}))
 
     def test_reprcall(self):
-        self.assertTrue(utils.reprcall("add",
-            (2, 2), {"copy": True}))
+        self.assertTrue(utils.reprcall('add',
+            (2, 2), {'copy': True}))
 
 
 class test_UUID(TestCase):
@@ -71,9 +83,9 @@ class test_UUID(TestCase):
 
     @skip_if_module('__pypy__')
     def test_uuid_without_ctypes(self):
-        old_utils = sys.modules.pop("kombu.utils")
+        old_utils = sys.modules.pop('kombu.utils')
 
-        @mask_modules("ctypes")
+        @mask_modules('ctypes')
         def with_ctypes_masked():
             from kombu.utils import ctypes, uuid
 
@@ -85,7 +97,7 @@ class test_UUID(TestCase):
         try:
             with_ctypes_masked()
         finally:
-            sys.modules["celery.utils"] = old_utils
+            sys.modules['celery.utils'] = old_utils
 
 
 class test_Misc(TestCase):
@@ -95,8 +107,8 @@ class test_Misc(TestCase):
         def f(**kwargs):
             return kwargs
 
-        kw = {u"foo": "foo",
-              u"bar": "bar"}
+        kw = {u'foo': 'foo',
+              u'bar': 'bar'}
         self.assertTrue(f(**utils.kwdict(kw)))
 
 
@@ -118,8 +130,8 @@ class test_emergency_dump_state(TestCase):
     def test_dump(self, stdout, stderr):
         fh = MyBytesIO()
 
-        utils.emergency_dump_state({"foo": "bar"}, open_file=lambda n, m: fh)
-        self.assertDictEqual(pickle.loads(fh.getvalue()), {"foo": "bar"})
+        utils.emergency_dump_state({'foo': 'bar'}, open_file=lambda n, m: fh)
+        self.assertDictEqual(pickle.loads(fh.getvalue()), {'foo': 'bar'})
         self.assertTrue(stderr.getvalue())
         self.assertFalse(stdout.getvalue())
 
@@ -128,9 +140,9 @@ class test_emergency_dump_state(TestCase):
         fh = MyStringIO()
 
         def raise_something(*args, **kwargs):
-            raise KeyError("foo")
+            raise KeyError('foo')
 
-        utils.emergency_dump_state({"foo": "bar"}, open_file=lambda n, m: fh,
+        utils.emergency_dump_state({'foo': 'bar'}, open_file=lambda n, m: fh,
                                                    dump=raise_something)
         self.assertIn("'foo': 'bar'", fh.getvalue())
         self.assertTrue(stderr.getvalue())
@@ -167,17 +179,30 @@ class test_retry_over_time(TestCase):
             raise self.Predicate()
         return 42
 
-    def errback(self, exc, interval):
+    def errback(self, exc, intervals, retries):
+        interval = next(intervals)
         sleepvals = (None, 2.0, 4.0, 6.0, 8.0, 10.0, 12.0, 14.0, 16.0, 16.0)
         self.index += 1
         self.assertEqual(interval, sleepvals[self.index])
+        return interval
 
     @insomnia
     def test_simple(self):
-        x = utils.retry_over_time(self.myfun, self.Predicate,
-                errback=self.errback, interval_max=14)
-        self.assertEqual(x, 42)
-        self.assertEqual(self.index, 9)
+        prev_count, utils.count = utils.count, Mock()
+        try:
+            utils.count.return_value = range(1)
+            x = utils.retry_over_time(self.myfun, self.Predicate,
+                    errback=None, interval_max=14)
+            self.assertIsNone(x)
+            utils.count.return_value = range(10)
+            cb = Mock()
+            x = utils.retry_over_time(self.myfun, self.Predicate,
+                    errback=self.errback, callback=cb, interval_max=14)
+            self.assertEqual(x, 42)
+            self.assertEqual(self.index, 9)
+            cb.assert_called_with()
+        finally:
+            utils.count = prev_count
 
     @insomnia
     def test_retry_once(self):
@@ -200,6 +225,26 @@ class test_retry_over_time(TestCase):
 
 class test_cached_property(TestCase):
 
+    def test_deleting(self):
+
+        class X(object):
+            xx = False
+
+            @utils.cached_property
+            def foo(self):
+                return 42
+
+            @foo.deleter  # noqa
+            def foo(self, value):
+                self.xx = value
+
+        x = X()
+        del(x.foo)
+        self.assertFalse(x.xx)
+        x.__dict__['foo'] = 'here'
+        del(x.foo)
+        self.assertEqual(x.xx, 'here')
+
     def test_when_access_from_class(self):
 
         class X(object):
@@ -213,7 +258,7 @@ class test_cached_property(TestCase):
             def foo(self, value):
                 self.xx = 10
 
-        desc = X.__dict__["foo"]
+        desc = X.__dict__['foo']
         self.assertIs(X.foo, desc)
 
         self.assertIs(desc.__get__(None), desc)
@@ -226,3 +271,75 @@ class test_cached_property(TestCase):
         self.assertEqual(x.xx, 10)
 
         del(x.foo)
+
+
+class test_symbol_by_name(TestCase):
+
+    def test_instance_returns_instance(self):
+        instance = object()
+        self.assertIs(utils.symbol_by_name(instance), instance)
+
+    def test_returns_default(self):
+        default = object()
+        self.assertIs(utils.symbol_by_name('xyz.ryx.qedoa.weq:foz',
+                        default=default), default)
+
+    def test_no_default(self):
+        with self.assertRaises(ImportError):
+            utils.symbol_by_name('xyz.ryx.qedoa.weq:foz')
+
+    def test_imp_reraises_ValueError(self):
+        imp = Mock()
+        imp.side_effect = ValueError()
+        with self.assertRaises(ValueError):
+            utils.symbol_by_name('kombu.Connection', imp=imp)
+
+    def test_package(self):
+        from kombu.entity import Exchange
+        self.assertIs(utils.symbol_by_name('.entity:Exchange',
+                    package='kombu'), Exchange)
+        self.assertTrue(utils.symbol_by_name(':Consumer', package='kombu'))
+
+
+class test_ChannelPromise(TestCase):
+
+    def test_repr(self):
+        self.assertEqual(repr(utils.ChannelPromise(lambda: 'foo')),
+                "<promise: 'foo'>")
+
+
+class test_entrypoints(TestCase):
+
+    @mask_modules('pkg_resources')
+    def test_without_pkg_resources(self):
+        self.assertListEqual(list(utils.entrypoints('kombu.test')), [])
+
+    @module_exists('pkg_resources')
+    def test_with_pkg_resources(self):
+        with patch('pkg_resources.iter_entry_points', create=True) as iterep:
+            eps = iterep.return_value = [Mock(), Mock()]
+
+            self.assertTrue(list(utils.entrypoints('kombu.test')))
+            iterep.assert_called_with('kombu.test')
+            eps[0].load.assert_called_with()
+            eps[1].load.assert_called_with()
+
+
+class test_shufflecycle(TestCase):
+
+    def test_shuffles(self):
+        prev_repeat, utils.repeat = utils.repeat, Mock()
+        try:
+            utils.repeat.return_value = range(10)
+            values = set(['A', 'B', 'C'])
+            cycle = utils.shufflecycle(values)
+            seen = set()
+            for i in xrange(10):
+                cycle.next()
+            utils.repeat.assert_called_with(None)
+            self.assertTrue(seen.issubset(values))
+            with self.assertRaises(StopIteration):
+                cycle.next()
+                cycle.next()
+        finally:
+            utils.repeat = prev_repeat
