@@ -9,10 +9,8 @@ from collections import defaultdict
 from itertools import count
 from Queue import Empty, Queue as _Queue
 
-from kombu.connection import Connection
-from kombu.entity import Exchange, Queue
+from kombu import Connection, Exchange, Queue, Consumer, Producer
 from kombu.exceptions import InconsistencyError, VersionMismatch
-from kombu.messaging import Consumer, Producer
 from kombu.utils import eventio  # patch poll
 
 from kombu.tests.utils import TestCase
@@ -195,6 +193,9 @@ class Channel(redis.Channel):
     def _get_client(self):
         return Client
 
+    def _get_pool(self):
+        return Mock()
+
     def _get_response_error(self):
         return ResponseError
 
@@ -380,10 +381,9 @@ class test_Channel(TestCase):
         c.connection.disconnect.assert_called_with()
 
     def test_invalid_database_raises_ValueError(self):
-        self.channel.connection.client.virtual_host = 'xfeqwewkfk'
 
         with self.assertRaises(ValueError):
-            self.channel._get_pool()
+            Connection('redis:///dwqwewqe').channel()
 
     @skip_if_not_module('redis')
     def test_get_client(self):
@@ -534,8 +534,7 @@ class test_Redis(TestCase):
                    transport=Transport).channel()
 
         with self.assertRaises(Exception):
-            Connection(virtual_host='/foo',
-                       transport=Transport).channel()
+            Connection('redis:///foo').channel()
 
     def test_db_port(self):
         c1 = Connection(port=None, transport=Transport).channel()
