@@ -4,9 +4,6 @@ kombu.transport.SQS
 
 Amazon SQS transport.
 
-:copyright: (c) 2010 - 2012 by Ask Solem
-:license: BSD, see LICENSE for more details.
-
 """
 from __future__ import absolute_import
 
@@ -25,7 +22,7 @@ from boto.sdb.connection import SDBConnection
 from boto.sqs.connection import SQSConnection
 from boto.sqs.message import Message
 
-from kombu.exceptions import StdChannelError
+from kombu.exceptions import StdConnectionError, StdChannelError
 from kombu.utils import cached_property, uuid
 from kombu.utils.encoding import safe_str
 
@@ -160,13 +157,14 @@ class Channel(virtual.Channel):
 
     def _new_queue(self, queue, **kwargs):
         """Ensures a queue exists in SQS."""
-        queue = self.queue_name_prefix + queue
+        # Translate to SQS name for consistency with initial
+        # _queue_cache population.
+        queue = self.entity_name(self.queue_name_prefix + queue)
         try:
             return self._queue_cache[queue]
         except KeyError:
             q = self._queue_cache[queue] = self.sqs.create_queue(
-                    self.entity_name(queue),
-                                        self.visibility_timeout)
+                    queue, self.visibility_timeout)
             return q
 
     def _queue_bind(self, *args):
@@ -346,5 +344,5 @@ class Transport(virtual.Transport):
 
     polling_interval = 1
     default_port = None
-    connection_errors = (exception.SQSError, socket.error)
+    connection_errors = (StdConnectionError, exception.SQSError, socket.error)
     channel_errors = (exception.SQSDecodeError, StdChannelError)
