@@ -6,9 +6,6 @@ kombu.transport.librabbitmq
 
 .. _`librabbitmq`: http://pypi.python.org/librabbitmq/
 
-:copyright: (c) 2010 - 2012 by Ask Solem.
-:license: BSD, see LICENSE for more details.
-
 """
 from __future__ import absolute_import
 
@@ -24,7 +21,7 @@ except ImportError:
     except ImportError:
         raise ImportError("No module named librabbitmq")
 
-from kombu.exceptions import StdChannelError
+from kombu.exceptions import StdConnectionError, StdChannelError
 from kombu.utils.amq_manager import get_manager
 
 from . import base
@@ -69,7 +66,8 @@ class Transport(base.Transport):
     Connection = Connection
 
     default_port = DEFAULT_PORT
-    connection_errors = (ConnectionError,
+    connection_errors = (StdConnectionError,
+                         ConnectionError,
                          socket.error,
                          IOError,
                          OSError)
